use crate::components::error::error_provider::ErrorContext;
use crate::services::rest::RestService;
use common::Achievement;
use common::ToggleAchievement;
use std::rc::Rc;
use wasm_bindgen_futures::spawn_local;
use yew::classes;
use yew::function_component;
use yew::html;
use yew::use_context;
use yew::Callback;
use yew::Html;
use yew::Properties;

#[derive(Properties, PartialEq)]
pub struct Props {
    pub achievement: Achievement,
    pub number: usize,
}

#[function_component]
pub fn AchievementComponent(props: &Props) -> Html {
    let err_ctx = Rc::new(use_context::<ErrorContext>());

    let Achievement {
        goal,
        completed,
        uuid,
        time_of_reveal: _,
    } = &props.achievement;

    let uuid = *uuid;

    let onclick_toggle = Callback::from(move |_| {
        log::info!("button click, toggling achievement");
        let err_ctx = Rc::clone(&err_ctx);
        spawn_local(async move {
            match RestService::toggle_achievement(ToggleAchievement { uuid }).await {
                Ok(_response) => {}
                Err(err) => {
                    if let Some(err_ctx) = &*err_ctx {
                        err_ctx.dispatch(err.to_string());
                    }
                }
            }
        });
    });

    let toggle_button_class = if *completed {
        "button-primary color-secondary"
    } else {
        "button"
    };

    html! {
        <div class="row flex">
            <div class="flex-intrinsic-size">
                <p>{format!("{}.", props.number)}</p>
            </div>
            <div class="flex-intrinsic-size">
                <button onclick={onclick_toggle} class={classes!(toggle_button_class)}><i class={classes!("fas", "fa-fw", completed.then_some("fa-star"), (!completed).then_some("fa-star-half-stroke"))} /></button>
            </div>
            <div class="flex-grow">
                <p>{goal}</p>
            </div>
        </div>
    }
}
