use crate::components::achievement::AchievementComponent;
use crate::components::milestone::MilestoneComponent;
use crate::util::group_achievements::group_achievements;
use yew::functional::*;
use yew::prelude::*;

#[function_component]
pub fn Root() -> Html {
    let app_state = use_context::<crate::AppState>().expect("no app state ctx found");

    let current_time: chrono::NaiveTime = chrono::Local::now().time();

    let achievements = app_state.state.achievements.clone();
    let grouped_achievements = group_achievements(achievements);

    let mut achievements_html: Vec<Html> = vec![];
    let mut n = 0;
    for (time_of_reveal, group) in grouped_achievements {
        if let Some(time_of_reveal) = time_of_reveal {
            let s = if time_of_reveal > current_time {
                format!(
                    "Next {} revealed at {}!",
                    group.len(),
                    time_of_reveal.format("%H:%M")
                )
            } else {
                format!("{}", time_of_reveal.format("%H:%M"))
            };
            let html = html! {
                <p style="text-align: center; font-weight: bold">{s}</p>
            };
            achievements_html.push(html);
            if time_of_reveal > current_time {
                break;
            }
        }
        for a in group {
            n += 1;
            let uuid = a.uuid.to_string();
            let html = html! {
                <AchievementComponent key={uuid} number={n} achievement={a} />
            };
            achievements_html.push(html);
        }
    }
    let achievements = achievements_html.into_iter().collect::<Html>();

    let completed_achievements = app_state
        .state
        .achievements
        .iter()
        .filter(|a| a.completed)
        .count();

    let mut milestones = app_state.state.milestones.clone();
    milestones.sort_by_key(|m| m.goal);
    let milestones = milestones.into_iter().map(|m| {let uuid = m.uuid.to_string(); html! { <MilestoneComponent key={uuid} milestone={m} completed_achievements={completed_achievements} /> }}).collect::<Html>();

    html! {
      <>

      <h1>{"Polter A"}<span style="color: var(--danger-color); text-decoration: line-through">{"bend"}</span>{"chievements!"}</h1>

      <hr />
      <div class="row flex flex-wrap">
        <div class="flex-grow">
          <h3><i style="margin-right: 0.5em" class="fas fa-trophy" />{"Milestones"}</h3>
        </div>
      </div>
      {milestones}

      <hr />
      <div class="row flex flex-wrap">
        <div class="flex-grow">
          <h3><i style="margin-right: 0.5em" class="fas fa-star" />{"Achievements"}</h3>
        </div>
      </div>
      {achievements}

      </>
    }
}
