use serde::de::DeserializeOwned;
use serde::Deserialize;
use serde::Serialize;
use std::collections::HashSet;
use yew_agent::HandlerId;
use yew_agent::Public;
use yew_agent::WorkerLink;

pub struct EventBus<T>
where
    T: 'static + Serialize + DeserializeOwned + Clone,
{
    link: WorkerLink<Self>,
    subscribers: HashSet<HandlerId>,
}

#[derive(Serialize, Deserialize)]
pub enum EventBusInput<T> {
    EventBusMsg(T),
}

impl<T> yew_agent::Worker for EventBus<T>
where
    T: 'static + Serialize + DeserializeOwned + Clone,
{
    type Reach = Public<Self>;
    type Input = EventBusInput<T>;
    type Output = T;
    type Message = ();

    fn create(link: WorkerLink<Self>) -> Self {
        log::debug!("Creating Event Bus.");
        Self {
            link,
            subscribers: HashSet::new(),
        }
    }

    fn update(&mut self, _msg: Self::Message) {}

    fn handle_input(&mut self, msg: Self::Input, _id: HandlerId) {
        match msg {
            EventBusInput::EventBusMsg(msg) => {
                log::debug!(
                    "Event bus received message. Responding to {} subscribers.",
                    self.subscribers.len()
                );
                for sub in &self.subscribers {
                    self.link.respond(*sub, msg.clone());
                }
            }
        }
    }

    fn connected(&mut self, id: HandlerId) {
        self.subscribers.insert(id);
    }

    fn disconnected(&mut self, id: HandlerId) {
        self.subscribers.remove(&id);
    }

    fn name_of_resource() -> &'static str {
        "event_bus.js"
    }
}
